@@ -19,8 +19,6 @@
     under the License.
 */
 
-/* $Id$ */
-
 #ifndef __mod_rivet_h__
 #define __mod_rivet_h__
 
@@ -38,10 +36,13 @@
 #endif
 
 /*
- * this macro replicates what AP_DECLARE_DATA does in mod_rivet.c but
- * it isn't meant to stay in mod_rivet.h in the first place
- * 
+ * Petasis 16 Dec 2018: This causes the symbol to be exported also from MPMs...
+ *
  * APLOG_USE_MODULE(rivet);
+ *
+ *  PLEASE: do not use any of APLOG_USE_MODULE, AP_DECLARE_MODULE,
+ *  AP_MODULE_DECLARE_DATA in this header file!
+ *
  */
 
 /* init.tcl file relative to the server root directory */
@@ -92,14 +93,13 @@
 
 /*
  * Petasis 10 Aug 2017: This causes the symbol to be exported also from MPMs...
-
-   module AP_MODULE_DECLARE_DATA rivet_module;
- 
-*/
+ *
+ *  module AP_MODULE_DECLARE_DATA rivet_module;
+ *
+ */
 
 typedef struct _rivet_server_conf {
 
-    char*       rivet_server_init_script;   /* run before children are forked  */
     char*       rivet_global_init_script;   /* run once when apache is started */
     char*       rivet_child_init_script;
     char*       rivet_child_exit_script;
@@ -119,28 +119,23 @@ typedef struct _rivet_server_conf {
 
     unsigned int user_scripts_status;
 
-    int             default_cache_size;
-    int             upload_max;
-    int             upload_files_to_var;
-    int             separate_virtual_interps;
-    int             honor_header_only_reqs;
-    int             separate_channels;      /* when true a vhosts get their private channel */
-    int             export_rivet_ns;        /* export the ::rivet namespace commands        */
-    int             import_rivet_ns;        /* import into the global namespace the
-                                               exported ::rivet commands                    */
-    char*           server_name;
-    const char*     upload_dir;
-    apr_table_t*    rivet_server_vars;
-    apr_table_t*    rivet_dir_vars;
-    apr_table_t*    rivet_user_vars;
-    int             idx;                /* server record index (to be used for the interps db)      */
-    char*           path;               /* copy of the path field of a cmd_parms structure:         
-                                         * should enable us to tell if a conf record comes from a
-                                         * Directory section */
-    const char*     mpm_bridge;         /* MPM bridge. if not null the module will try to load the      * 
-                                         * file name in this field. The string should be either a full  *
-                                         * path to a file name, or a string from which a file name will *
-                                         * be composed using the pattern 'rivet_(mpm_bridge)_mpm.so     */
+    int          default_cache_size;
+    int          upload_max;
+    int          upload_files_to_var;
+    int          honor_header_only_reqs;
+
+    int          export_rivet_ns;        /* export the ::rivet namespace commands        */
+    int          import_rivet_ns;        /* import into the global namespace the
+                                            exported ::rivet commands                    */
+    char*        server_name;
+    const char*  upload_dir;
+    apr_table_t* rivet_server_vars;
+    apr_table_t* rivet_dir_vars;
+    apr_table_t* rivet_user_vars;
+    int          idx;                   /* server record index (to be used for the interps db)    */
+    char*        path;                  /* copy of the path field of a cmd_parms structure:       *
+                                         * should enable us to tell if a conf record comes from a *
+                                         * Directory section                                      */
 } rivet_server_conf;
 
 #define TCL_INTERPS 1
@@ -175,20 +170,20 @@ typedef struct _rivet_thread_interp {
 typedef struct _thread_worker_private rivet_thread_private;
 
 typedef int                     (RivetBridge_ServerInit)    (apr_pool_t*,apr_pool_t*,apr_pool_t*,server_rec*);
-typedef void                    (RivetBridge_ChildInit)     (apr_pool_t* pPool,server_rec* s);
+typedef void                    (RivetBridge_ThreadInit)    (apr_pool_t* pPool,server_rec* s);
 typedef int                     (RivetBridge_Request)       (request_rec*,rivet_req_ctype);
 typedef apr_status_t            (RivetBridge_Finalize)      (void*);
 typedef rivet_thread_interp*    (RivetBridge_Master_Interp) (void);
-typedef int                     (RivetBridge_Exit_Handler)  (int);
+typedef int                     (RivetBridge_Exit_Handler)  (rivet_thread_private*);
 typedef rivet_thread_interp*    (RivetBridge_Thread_Interp) (rivet_thread_private*,rivet_server_conf *,rivet_thread_interp*);
 
 typedef struct _mpm_bridge_table {
-    RivetBridge_ServerInit    *mpm_server_init;
-    RivetBridge_ChildInit     *mpm_child_init;
-    RivetBridge_Request       *mpm_request;
-    RivetBridge_Finalize      *mpm_finalize;
-    RivetBridge_Exit_Handler  *mpm_exit_handler;
-    RivetBridge_Thread_Interp *mpm_thread_interp;
+    RivetBridge_ServerInit    *server_init;
+    RivetBridge_ThreadInit    *thread_init;
+    RivetBridge_Request       *request_processor;
+    RivetBridge_Finalize      *child_finalize;
+    RivetBridge_Exit_Handler  *exit_handler;
+    RivetBridge_Thread_Interp *thread_interp;
 } rivet_bridge_table;
 
 /* we need also a place where to store globals with module wide scope */
@@ -199,15 +194,28 @@ typedef struct _mod_rivet_globals {
     apr_pool_t*         pool;               
     char*               rivet_mpm_bridge;       /* name of the MPM bridge                   */
     server_rec*         server;                 /* default host server_rec obj              */
-    int                 vhosts_count;           /* Number of configured virtual host including 
+    int                 vhosts_count;           /* Number of configured virtual host including   *
                                                  * the root server thus it's supposed to be >= 1 */
-	char*				default_handler;		/* Default request handler code             */
-	int					default_handler_size;	/* Size of the default_handler buffer       */
+    char*				default_handler;		/* Default request handler code             */
+    int					default_handler_size;	/* Size of the default_handler buffer       */
     rivet_thread_interp* 
                         server_interp;          /* server and prefork MPM interpreter       */
     apr_thread_mutex_t* pool_mutex;             /* threads commmon pool mutex               */
     rivet_bridge_table* bridge_jump_table;      /* Jump table to bridge specific procedures */
+    const char*         rivet_server_init_script; /* run before children are forked         */
+    const char*         mpm_bridge;             /* MPM bridge. if not null the module will  */
+                                                /* try to load the file name in this field. */
+                                                /* The string should be either a full       */
+                                                /* path to a file name, or a string from    */
+                                                /* which a file name will be composed using */
+                                                /* the pattern 'rivet_(mpm_bridge)_mpm.so   */
     mpm_bridge_status*  mpm;                    /* bridge private control structure         */
+    int                 single_thread_exit;     /* With a threaded bridge allow a single    */
+                                                /* thread to exit instead of forcing the    */
+                                                /* whole process to terminate               */
+    int                 separate_virtual_interps; 
+                                                /* Virtual host have their own interpreter  */
+    int                 separate_channels;      /* when true a vhosts get their private channel */
 #ifdef RIVET_SERIALIZE_HTTP_REQUESTS
     apr_thread_mutex_t* req_mutex;
 #endif
@@ -222,8 +230,6 @@ typedef struct _thread_worker_private {
     rivet_req_ctype     ctype;              /*                                      */
     request_rec*        r;                  /* current request_rec                  */
     TclWebRequest*      req;
-    //Tcl_Obj*          request_init;
-    Tcl_Obj*            request_cleanup;
     rivet_server_conf*  running_conf;       /* running configuration                */
     running_scripts*    running;            /* (per request) running conf scripts   */
     int                 thread_exit;        /* Thread exit code                     */
@@ -287,6 +293,12 @@ Tcl_Obj* Rivet_CurrentServerRec (Tcl_Interp* interp, server_rec* s);
 #define ABORTPAGE_CODE              "ABORTPAGE"
 #define THREAD_EXIT_CODE            "THREAD_EXIT"
 
+/* Configuration defaults */
+
+#define SINGLE_THREAD_EXIT_UNDEF   -1    /* pre config undefined value for single 
+                                            thread exit flag in the module globals
+                                            structure */
+
 #define TCL_MAX_CHANNEL_BUFFER_SIZE (1024*1024)
 #define MODNAME                     "mod_rivet"
 
@@ -310,6 +322,11 @@ Tcl_Obj* Rivet_CurrentServerRec (Tcl_Interp* interp, server_rec* s);
         Tcl_IncrRefCount(running_script->objscript);\
     }
 
+#define RIVET_SCRIPT_DISPOSE(running_scripts,script_name) \
+    if (running_scripts->script_name != NULL) {\
+        Tcl_DecrRefCount(running_scripts->script_name);\
+    }
+
 #define RIVET_MPM_BRIDGE_TABLE         bridge_jump_table
 #define RIVET_MPM_BRIDGE_FUNCTION(fun) module_globals->bridge_jump_table->fun
 
@@ -318,11 +335,13 @@ Tcl_Obj* Rivet_CurrentServerRec (Tcl_Interp* interp, server_rec* s);
 }
 
 #define RIVET_PEEK_INTERP(thread_private,running_conf) \
-        (module_globals->bridge_jump_table->mpm_thread_interp)(thread_private,running_conf,NULL)
+        (module_globals->bridge_jump_table->thread_interp)(thread_private,running_conf,NULL)
 
 #define RIVET_POKE_INTERP(thread_private,running_conf,interp) \
-        (module_globals->bridge_jump_table->mpm_thread_interp)(thread_private,running_conf,interp)
+        (module_globals->bridge_jump_table->thread_interp)(thread_private,running_conf,interp)
 
 #define RIVET_MPM_BRIDGE rivet_bridge_table bridge_jump_table =
 
+#define RIVET_MPM_BRIDGE_COMPOSE(bridge) RIVET_DIR,"/mpm/rivet_",bridge,"_mpm.so"
+
 #endif /* MOD_RIVET_H */
