@@ -21,14 +21,12 @@
     under the License.
 */
 
-/* $Id$ */
-
 #include <httpd.h>
 #include <apr_strings.h>
 #include <apr_env.h>
-#include <ap_mpm.h>
 #include <apr_file_io.h>
 #include <apr_file_info.h>
+#include <ap_mpm.h>
 
 #include "mod_rivet.h"
 #include "rivetChannel.h"
@@ -36,7 +34,6 @@
 #include "rivetParser.h"
 #include "rivet.h"
 #include "apache_config.h"
-#include <mpm_common.h>
 
 /* as long as we need to emulate ap_chdir_file we need to include unistd.h */
 #ifdef RIVET_HAVE_UNISTD_H
@@ -98,12 +95,11 @@ Rivet_ReadFile (apr_pool_t* pool,char* filename,
 
     apr_file_close(apr_fp);
  
-    *nbytes = buffer_size;
+    *nbytes = (int)buffer_size;
     return 0;
 }
 
-/*
- *-----------------------------------------------------------------------------
+/*-----------------------------------------------------------------------------
  * Rivet_CreateTclInterp --
  *
  * Arguments:
@@ -118,7 +114,7 @@ Rivet_ReadFile (apr_pool_t* pool,char* filename,
  */
 
 static Tcl_Interp* 
-Rivet_CreateTclInterp (server_rec* s)
+Rivet_CreateTclInterp (apr_pool_t* pool)
 {
     Tcl_Interp* interp;
 
@@ -128,14 +124,14 @@ Rivet_CreateTclInterp (server_rec* s)
 
     if (interp == NULL)
     {
-        ap_log_error(APLOG_MARK, APLOG_ERR, APR_EGENERAL, s,
+        ap_log_perror(APLOG_MARK, APLOG_ERR, APR_EGENERAL, pool,
                      MODNAME ": Error in Tcl_CreateInterp, aborting\n");
         exit(1);
     }
 
     if (Tcl_Init(interp) == TCL_ERROR)
     {
-        ap_log_error(APLOG_MARK, APLOG_ERR, APR_EGENERAL, s,
+        ap_log_perror(APLOG_MARK, APLOG_ERR, APR_EGENERAL, pool,
                      MODNAME ": Error in Tcl_Init: %s, aborting\n",
                      Tcl_GetStringResult(interp));
         exit(1);
@@ -183,6 +179,43 @@ running_scripts* Rivet_RunningScripts ( apr_pool_t* pool,
     return scripts;
 }
 
+/*
+ *  -- Rivet_ReleaseRunningScripts 
+ *
+ */
+
+void Rivet_ReleaseRunningScripts (running_scripts* scripts)
+{
+    RIVET_SCRIPT_DISPOSE(scripts,rivet_before_script);
+    RIVET_SCRIPT_DISPOSE(scripts,rivet_after_script);
+    RIVET_SCRIPT_DISPOSE(scripts,rivet_error_script);
+    RIVET_SCRIPT_DISPOSE(scripts,rivet_abort_script);
+    RIVET_SCRIPT_DISPOSE(scripts,after_every_script);
+    RIVET_SCRIPT_DISPOSE(scripts,request_processing);
+}
+
+/*
+ * -- Rivet_ReleasePerDirScripts
+ *
+ */
+
+void Rivet_ReleasePerDirScripts(rivet_thread_interp* rivet_interp)
+{
+    apr_hash_t*         ht = rivet_interp->per_dir_scripts;
+    apr_hash_index_t*   hi;
+    Tcl_Obj*            script;
+    apr_pool_t*         p = rivet_interp->pool;
+
+    for (hi = apr_hash_first(p,ht); hi; hi = apr_hash_next(hi))
+    {
+        apr_hash_this(hi, NULL, NULL, (void*)(&script));
+        Tcl_DecrRefCount(script);
+    }
+
+    apr_hash_clear(ht);
+}
+
+
 /*
  *---------------------------------------------------------------------
  *
@@ -301,46 +334,35 @@ void Rivet_PerInterpInit(rivet_thread_interp* interp_obj,
   *
   */
 
-rivet_thread_interp* Rivet_NewVHostInterp(apr_pool_t *pool,server_rec* server)
+rivet_thread_interp* Rivet_NewVHostInterp(apr_pool_t *pool,int default_cache_size)
 {
     rivet_thread_interp*    interp_obj = apr_pcalloc(pool,sizeof(rivet_thread_interp));
-    rivet_server_conf*      rsc;
-
-    /* The cache size is global so we take it from here */
-    
-    rsc = RIVET_SERVER_CONF (server->module_config);
 
     /* This calls needs the root server_rec just for logging purposes */
 
-    interp_obj->interp = Rivet_CreateTclInterp(server); 
+    interp_obj->interp = Rivet_CreateTclInterp(pool); 
+
+    /* we now create memory from the cache pool as subpool of the thread private pool */
+ 
+    if (apr_pool_create(&interp_obj->pool, pool) != APR_SUCCESS)
+    {
+        ap_log_perror(APLOG_MARK, APLOG_ERR, APR_EGENERAL, pool, 
+                     MODNAME ": could not initialize cache private pool");
+        return NULL;
+    }
 
     /* We now read from the pointers to the cache_size and cache_free conf parameters
      * for compatibility with mod_rivet current version, but these values must become
      * integers not pointers
      */
     
-    if (rsc->default_cache_size < 0) {
-        if (ap_max_requests_per_child != 0) {
-            interp_obj->cache_size = ap_max_requests_per_child / 5;
-        } else {
-            interp_obj->cache_size = 50;    // Arbitrary number
-        }
-    } else if (rsc->default_cache_size > 0) {
-        interp_obj->cache_size = rsc->default_cache_size;
+    if (default_cache_size < 0) {
+        interp_obj->cache_size = RivetCache_DefaultSize();
+    } else if (default_cache_size > 0) {
+        interp_obj->cache_size = default_cache_size;
     }
 
-    if (interp_obj->cache_size > 0) {
-        interp_obj->cache_free = interp_obj->cache_size;
-    }
-
-    /* we now create memory from the cache pool as subpool of the thread private pool */
- 
-    if (apr_pool_create(&interp_obj->pool, pool) != APR_SUCCESS)
-    {
-        ap_log_error(APLOG_MARK, APLOG_ERR, APR_EGENERAL, server, 
-                     MODNAME ": could not initialize cache private pool");
-        return NULL;
-    }
+    interp_obj->cache_free = interp_obj->cache_size;
 
     // Initialize cache structures
 
