@@ -19,8 +19,6 @@
     under the License.
 */
 
-/* $Id$ */
-
 #include <httpd.h>
 #include <tcl.h>
 #include <apr_strings.h>
@@ -113,7 +111,7 @@ Rivet_ReleaseScripts (running_scripts* scripts)
 #define USE_APACHE_RSC
 
 DLLEXPORT int
-Rivet_SendContent(rivet_thread_private *private,request_rec* r)
+Rivet_SendContent(rivet_thread_private *private)
 {
     int                     errstatus;
     int                     retval;
@@ -121,14 +119,6 @@ Rivet_SendContent(rivet_thread_private *private,request_rec* r)
     rivet_thread_interp*    interp_obj;
     Tcl_Channel*            running_channel;
 
-#ifdef USE_APACHE_RSC
-    //rivet_server_conf    *rsc = NULL;
-#else
-    //rivet_server_conf    *rdc;
-#endif
-
-    private->r = r;
-
     /* Set the global request req to know what we are dealing with in
      * case we have to call the PanicProc. */
 
@@ -146,7 +136,7 @@ Rivet_SendContent(rivet_thread_private *private,request_rec* r)
 
     interp_obj = RIVET_PEEK_INTERP(private,private->running_conf);
     private->running = interp_obj->scripts;
-    running_channel = interp_obj->channel;
+    running_channel  = interp_obj->channel;
 
     if (private->r->per_dir_config)
     {
@@ -183,7 +173,7 @@ Rivet_SendContent(rivet_thread_private *private,request_rec* r)
 
         if (USER_CONF_UPDATED(rdc))
         {
-            rivet_server_conf*  newconfig   = NULL;
+            rivet_server_conf* newconfig = NULL;
             private->running = (running_scripts *) apr_pcalloc (private->pool,sizeof(running_scripts));
 
             newconfig = RIVET_NEW_CONF(private->r->pool);
@@ -193,7 +183,6 @@ Rivet_SendContent(rivet_thread_private *private,request_rec* r)
             private->running_conf = newconfig;
 
             private->running = Rivet_RunningScripts(private->r->pool,private->running,newconfig);
-
         }
     }
     else
@@ -263,20 +252,6 @@ Rivet_SendContent(rivet_thread_private *private,request_rec* r)
         goto sendcleanup;
     }
 
-    /*
-    if (Tcl_EvalObjEx(interp, private->request_init, 0) == TCL_ERROR)
-    {
-        request_rec* r = private->r;
-
-        ap_log_error(APLOG_MARK, APLOG_ERR, APR_EGENERAL, r->server,
-                            MODNAME ": Could not create request namespace (%s)\n" ,
-                            Tcl_GetStringResult(interp));
-
-        retval = HTTP_INTERNAL_SERVER_ERROR;
-        goto sendcleanup;
-    }
-    */
-
     /* Apache Request stuff */
 
     TclWeb_InitRequest(private, interp);
@@ -315,8 +290,6 @@ Rivet_SendContent(rivet_thread_private *private,request_rec* r)
     /* URL referenced script execution and exception handling */
 
     if (Tcl_EvalObjEx(interp, private->running->request_processing,0) == TCL_ERROR) 
-    //if (Rivet_ParseExecFile (private, private->r->filename, 1) != TCL_OK)
-    //if (Rivet_ExecuteAndCheck(private,private->request_processing) == TCL_ERROR)
     {
         /* we don't report errors coming from abort_page execution */
 
@@ -338,18 +311,8 @@ Rivet_SendContent(rivet_thread_private *private,request_rec* r)
         private->running_conf->user_scripts_status &= ~(unsigned int)USER_SCRIPTS_UPDATED;
     }
 
-    /* and finally we run the request_cleanup procedure (always set) */
-    
-    //if (Tcl_EvalObjEx(interp, private->request_cleanup, 0) == TCL_ERROR) {
-    //
-    //    ap_log_rerror(APLOG_MARK, APLOG_ERR, APR_EGENERAL, private->r, 
-    //                 MODNAME ": Error evaluating cleanup request: %s",
-    //                 Tcl_GetVar(interp, "errorInfo", 0));
-    //
-    //}
-
-    /* We finalize the request processing by printing the headers and flushing
-       the rivet channel internal buffer */
+    /* We finalize the request processing by printing the headers
+     * and flushing the rivet channel internal buffer */
 
     TclWeb_PrintHeaders(private->req);
     Tcl_Flush(*(running_channel));
@@ -370,11 +333,12 @@ sendcleanup:
     
     if (private->thread_exit)
     {
-        ap_log_rerror(APLOG_MARK, APLOG_DEBUG, APR_SUCCESS, private->r, 
-                                  "process terminating with code %d",private->exit_status);
-        RIVET_MPM_BRIDGE_CALL(mpm_exit_handler,private->exit_status);
+        ap_log_rerror(APLOG_MARK,APLOG_DEBUG,APR_SUCCESS,private->r, 
+                                "process terminating with code %d",private->exit_status);
+        RIVET_MPM_BRIDGE_CALL(exit_handler,private);
+
         //Tcl_Exit(private->exit_status);
-        exit(private->exit_status);
+        //exit(private->exit_status);
     }
 
     /* We now reset the status to prepare the child process for another request */
