@@ -1,5 +1,5 @@
 /* -- rivet_types.h: this file should collect all the basic types used
- * in mod_rivet and other related code
+ *    in mod_rivet and other related code
  */
 
 /*
@@ -21,14 +21,35 @@
     under the License.
  */
 
-/* $Id$ */
-
-#ifndef _RIVET_TYPES_H_
-#define _RIVET_TYPES_H_
+#ifndef __rivet_types_h__
+#define __rivet_types_h__
 
 #include <httpd.h>
 #include <tcl.h>
 
+/* Definition suggested in
+ *
+ * https://www.gnu.org/software/autoconf/manual/autoconf-2.67/html_node/Particular-Headers.html
+ *
+ * in order to have a portable definition of the 'bool' data type
+ */
+
+#ifdef HAVE_STDBOOL_H
+# include <stdbool.h>
+#else
+# ifndef HAVE__BOOL
+#  ifdef __cplusplus
+typedef bool _Bool;
+#  else
+#   define _Bool signed char
+#  endif
+# endif
+# define bool _Bool
+# define false 0
+# define true 1
+# define __bool_true_false_are_defined 1
+#endif
+
 typedef struct _ApacheRequest   ApacheRequest;
 typedef struct _ApacheUpload    ApacheUpload;
 
@@ -39,7 +60,7 @@ typedef struct _ApacheUpload {
     char*           tempname;
     apr_table_t*    info;
     apr_file_t*     fp;
-    long            size;
+    size_t          size;
     ApacheRequest*  req;
 } ApacheUpload;
 
@@ -53,7 +74,7 @@ typedef struct _ApacheRequest {
     int             (*upload_hook)(void *ptr, char *buf, int len, ApacheUpload *upload);
     void*           hook_data;
     const char*     temp_dir;
-    char*           raw_post; /* Raw post data. */
+    char*           raw_post;           /* Raw post data. */
     request_rec*    r;
     int             nargs;
 } ApacheRequest;
@@ -64,10 +85,11 @@ typedef struct TclWebRequest {
     ApacheRequest*  apachereq;
     ApacheUpload*   upload;
     int             headers_printed;	/* has the header been printed yet? */
-    int             headers_set;		/* has the header been set yet? */
+    int             headers_set;		/* has the header been set yet?     */
     int             content_sent;
     int             environment_set;	/* have we setup the environment variables? */
     char*           charset;
 } TclWebRequest;
 
-#endif
+#endif /* __rivet_types_h__ */
+
