@@ -19,11 +19,12 @@
     under the License.
 */
 
-/* $Id$ */
-
 #include <httpd.h>
 #include <apr_strings.h>
+#include <ap_mpm.h>
+#include <mpm_common.h>
 #include "mod_rivet.h"
+
 /* Function prototypes are defined with EXTERN. Since we are in the same DLL,
  * no need to keep this extern... */
 #ifdef EXTERN
@@ -36,7 +37,7 @@
 
 extern DLLIMPORT mod_rivet_globals* module_globals;
 extern DLLIMPORT apr_threadkey_t*   rivet_thread_key;
-extern DLLIMPORT module rivet_module;
+extern DLLIMPORT module             rivet_module;
 
 extern rivet_thread_interp* MPM_MasterInterp(server_rec* s);
 
@@ -104,6 +105,19 @@ rivet_thread_private* Rivet_VirtualHostsInterps (rivet_thread_private* private)
     /* we must assume the module was able to create the root interprter */ 
 
     ap_assert (root_interp != NULL);
+    
+    /* The inherited interpreter has an empty cache since evalutating a server_init_script
+     * does not require parsing templates that need to be stored in it. We need to
+     * create it
+     */
+
+    if (root_server_conf->default_cache_size > 0) {
+        root_interp->cache_size = root_server_conf->default_cache_size;
+    } else if (root_server_conf->default_cache_size < 0) {
+        root_interp->cache_size = RivetCache_DefaultSize();
+    }
+
+    RivetCache_Create(root_interp->pool,root_interp); 
 
     /* Using the root interpreter we evaluate the global initialization script, if any */
 
@@ -151,10 +165,10 @@ rivet_thread_private* Rivet_VirtualHostsInterps (rivet_thread_private* private)
         }
         else 
         {
-            if (root_server_conf->separate_virtual_interps)
+            if (module_globals->separate_virtual_interps)
             {
-                rivet_interp = Rivet_NewVHostInterp(private->pool,s);
-                if (myrsc->separate_channels)
+                rivet_interp = Rivet_NewVHostInterp(private->pool,myrsc->default_cache_size);
+                if (module_globals->separate_channels)
                 {
                     rivet_interp->channel = Rivet_CreateRivetChannel(private->pool,rivet_thread_key);
                     Tcl_RegisterChannel(rivet_interp->interp,*rivet_interp->channel);
@@ -202,7 +216,7 @@ rivet_thread_private* Rivet_VirtualHostsInterps (rivet_thread_private* private)
 
         function = myrsc->rivet_child_init_script;
         if (function && 
-            (s == root_server || root_server_conf->separate_virtual_interps || function != parentfunction))
+            (s == root_server || module_globals->separate_virtual_interps || function != parentfunction))
         {
             char*       errmsg = MODNAME ": Error in Child init script: %s";
             Tcl_Interp* interp = rivet_interp->interp;
@@ -271,7 +285,6 @@ void Rivet_ProcessorCleanup (void *data)
 {
     int                     i;
     rivet_thread_private*   private = (rivet_thread_private *) data;
-    rivet_server_conf*      rsc = RIVET_SERVER_CONF(module_globals->server->module_config);
 
     ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, module_globals->server, 
                  "Thread exiting after %d requests served (%d vhosts)", 
@@ -298,20 +311,25 @@ void Rivet_ProcessorCleanup (void *data)
 
         RivetCache_Cleanup(private,private->ext->interps[i]);
 
-        if ((i > 0) && rsc->separate_channels) 
+        if ((i > 0) && module_globals->separate_channels) 
             Rivet_ReleaseRivetChannel(private->ext->interps[i]->interp,private->channel);
 
         Tcl_DeleteInterp(private->ext->interps[i]->interp);
 
+        /* Release interpreter scripts */
+
+        Rivet_ReleaseRunningScripts(private->ext->interps[i]->scripts);
+
+        /* Release scripts defined within <Directory...></Directory> conf blocks */
+
+        Rivet_ReleasePerDirScripts(private->ext->interps[i]);
+
         /* if separate_virtual_interps == 0 we are running the same interpreter
          * instance for each vhost, thus we can jump out of this loop after 
          * the first cycle as the only real intepreter object we have is stored
          * in private->ext->interps[0]
          */
 
-    } while ((++i < module_globals->vhosts_count) && rsc->separate_virtual_interps);
+    } while ((++i < module_globals->vhosts_count) && module_globals->separate_virtual_interps);
 
-    //Tcl_DecrRefCount(private->request_init);
-    //Tcl_DecrRefCount(private->request_cleanup);
-    apr_pool_destroy(private->pool);
 }
